import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Input,
  OnInit,
} from "@angular/core";
import {
  ControlValueAccessor,
  FormControl,
  NG_VALUE_ACCESSOR,
} from "@angular/forms";
import {Enums} from "@enums";
import {environment} from "@environments/environment";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {KeyValueInfo} from "@shared/models/key-value-info.model";
import {tap} from "rxjs/operators";

@Component({
  selector: "dlcm-shared-data-sensitivity-input",
  templateUrl: "./shared-data-sensitivity-input.presentational.html",
  styleUrls: ["./shared-data-sensitivity-input.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: SharedDataSensitivityInputPresentational,
    },
  ],
})
export class SharedDataSensitivityInputPresentational extends SharedAbstractPresentational implements OnInit, ControlValueAccessor {
  readonly TIME_BEFORE_DISPLAY_TOOLTIP: number = environment.timeBeforeDisplayTooltipOnInput;

  @Input()
  formControl: FormControl;

  ngOnInit(): void {
    super.ngOnInit();

    this.subscribe(this.formControl.statusChanges.pipe(
      tap(status => {
        this._changeDetector.detectChanges();
      }),
    ));
  }

  dataSensitivityEnumValues: KeyValueInfo[] = Enums.Deposit.DataSensitivityEnumTranslate;

  get dataSensitivityEnum(): typeof Enums.Deposit.DataSensitivityEnum {
    return Enums.Deposit.DataSensitivityEnum;
  }

  constructor(private readonly _changeDetector: ChangeDetectorRef) {
    super();
  }

  registerOnChange(fn: any): void {
    this.propagateChange = fn;
  }

  propagateChange = (__: any) => {};

  registerOnTouched(fn: any): void {
  }

  writeValue(obj: any): void {
  }
}
